package sun.security.x509;

import java.util.Arrays;
import sun.misc.HexDumpEncoder;
import sun.security.util.DerOutputStream;
import sun.security.util.BitArray;
import java.net.InetAddress;
import java.io.IOException;
import sun.security.util.DerValue;

public class IPAddressName implements GeneralNameInterface
{
    private byte[] address;
    private boolean isIPv4;
    private String name;
    private static final int MASKSIZE = 16;
    
    public IPAddressName(DerValue derValue) throws IOException {
        this(derValue.getOctetString());
    }
    
    public IPAddressName(byte[] address) throws IOException {
        if (address.length == 4 || address.length == 8) {
            this.isIPv4 = true;
        }
        else if (address.length == 16 || address.length == 32) {
            this.isIPv4 = false;
        }
        else {
            throw new IOException("Invalid IPAddressName");
        }
        this.address = address;
    }
    
    public IPAddressName(String s) throws IOException {
        if (s == null || s.length() == 0) {
            throw new IOException("IPAddress cannot be null or empty");
        }
        if (s.charAt(s.length() - 1) == '/') {
            throw new IOException("Invalid IPAddress: " + s);
        }
        if (s.indexOf(58) >= 0) {
            this.parseIPv6(s);
            this.isIPv4 = false;
        }
        else if (s.indexOf(46) >= 0) {
            this.parseIPv4(s);
            this.isIPv4 = true;
        }
        else {
            throw new IOException("Invalid IPAddress: " + s);
        }
        return;
    }
    
    private void parseIPv4(String host) throws IOException {
        int index = host.indexOf(47);
        if (index == -1) {
            this.address = InetAddress.getByName(host).getAddress();
        }
        else {
            this.address = new byte[8];
            byte[] address = InetAddress.getByName(host.substring(index + 1)).getAddress();
            System.arraycopy(InetAddress.getByName(host.substring(0, index)).getAddress(), 0, this.address, 0, 4);
            System.arraycopy(address, 0, this.address, 4, 4);
        }
    }
    
    private void parseIPv6(String host) throws IOException {
        int index = host.indexOf(47);
        if (index == -1) {
            this.address = InetAddress.getByName(host).getAddress();
        }
        else {
            this.address = new byte[32];
            System.arraycopy(InetAddress.getByName(host.substring(0, index)).getAddress(), 0, this.address, 0, 16);
            int int1 = Integer.parseInt(host.substring(index + 1));
            if (int1 < 0 || int1 > 128) {
                throw new IOException("IPv6Address prefix length (" + int1 + ") in out of valid range [0,128]");
            }
            BitArray bitArray = new BitArray(128);
            for (int i = 0; i < int1; ++i) {
                bitArray.set(i, true);
            }
            byte[] byteArray = bitArray.toByteArray();
            for (int j = 0; j < 16; ++j) {
                this.address[16 + j] = byteArray[j];
            }
        }
    }
    
    @Override
    public int getType() {
        return 7;
    }
    
    @Override
    public void encode(DerOutputStream derOutputStream) throws IOException {
        derOutputStream.putOctetString(this.address);
    }
    
    @Override
    public String toString() {
        try {
            return "IPAddress: " + this.getName();
        }
        catch (IOException ex) {
            return "IPAddress: " + new HexDumpEncoder().encodeBuffer(this.address);
        }
    }
    
    public String getName() throws IOException {
        if (this.name != null) {
            return this.name;
        }
        if (this.isIPv4) {
            byte[] addr2 = new byte[4];
            System.arraycopy(this.address, 0, addr2, 0, 4);
            this.name = InetAddress.getByAddress(addr2).getHostAddress();
            if (this.address.length == 8) {
                byte[] addr3 = new byte[4];
                System.arraycopy(this.address, 4, addr3, 0, 4);
                this.name = this.name + "/" + InetAddress.getByAddress(addr3).getHostAddress();
            }
        }
        else {
            byte[] addr = new byte[16];
            System.arraycopy(this.address, 0, addr, 0, 16);
            this.name = InetAddress.getByAddress(addr).getHostAddress();
            if (this.address.length == 32) {
                byte[] a = new byte[16];
                for (int i = 16; i < 32; ++i) {
                    a[i - 16] = this.address[i];
                }
                BitArray bitArray;
                int j;
                for (bitArray = new BitArray(128, a), j = 0; j < 128 && bitArray.get(j); ++j) {}
                this.name = this.name + "/" + j;
                while (j < 128) {
                    if (bitArray.get(j)) {
                        throw new IOException("Invalid IPv6 subdomain - set bit " + j + " not contiguous");
                    }
                    ++j;
                }
            }
        }
        return this.name;
    }
    
    public byte[] getBytes() {
        return (byte[])this.address.clone();
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof IPAddressName)) {
            return false;
        }
        byte[] address = ((IPAddressName)o).address;
        if (address.length != this.address.length) {
            return false;
        }
        if (this.address.length == 8 || this.address.length == 32) {
            int i = this.address.length / 2;
            int j = 0;
            while (j < i) {
                if ((byte)(this.address[j] & this.address[j + i]) != (byte)(address[j] & address[j + i])) {
                    return false;
                }
                ++j;
            }
            while (i < this.address.length) {
                if (this.address[i] != address[i]) {
                    return false;
                }
                ++i;
            }
            return true;
        }
        return Arrays.equals(address, this.address);
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        for (int i = 0; i < this.address.length; ++i) {
            n += this.address[i] * i;
        }
        return n;
    }
    
    @Override
    public int constrains(GeneralNameInterface generalNameInterface) throws UnsupportedOperationException {
        int n;
        if (generalNameInterface == null) {
            n = -1;
        }
        else if (generalNameInterface.getType() != 7) {
            n = -1;
        }
        else if (((IPAddressName)generalNameInterface).equals(this)) {
            n = 0;
        }
        else {
            byte[] address = ((IPAddressName)generalNameInterface).address;
            if (address.length == 4 && this.address.length == 4) {
                n = 3;
            }
            else if ((address.length == 8 && this.address.length == 8) || (address.length == 32 && this.address.length == 32)) {
                boolean n6 = true;
                boolean n7 = true;
                boolean n8 = false;
                boolean n9 = false;
                for (int n10 = this.address.length / 2, i = 0; i < n10; ++i) {
                    if ((byte)(this.address[i] & this.address[i + n10]) != this.address[i]) {
                        n8 = true;
                    }
                    if ((byte)(address[i] & address[i + n10]) != address[i]) {
                        n9 = true;
                    }
                    if ((byte)(this.address[i + n10] & address[i + n10]) != this.address[i + n10] || (byte)(this.address[i] & this.address[i + n10]) != (byte)(address[i] & this.address[i + n10])) {
                        n6 = false;
                    }
                    if ((byte)(address[i + n10] & this.address[i + n10]) != address[i + n10] || (byte)(address[i] & address[i + n10]) != (byte)(this.address[i] & address[i + n10])) {
                        n7 = false;
                    }
                }
                if (n8 || n9) {
                    if (n8 && n9) {
                        n = 0;
                    }
                    else if (!(n8 == 0)) {
                        n = 2;
                    }
                    else {
                        n = 1;
                    }
                }
                else if (!(n6 == 0)) {
                    n = 1;
                }
                else if (!(n7 == 0)) {
                    n = 2;
                }
                else {
                    n = 3;
                }
            }
            else if (address.length == 8 || address.length == 32) {
                int n4;
                int n5;
                for (n4 = 0, n5 = address.length / 2; n4 < n5 && (this.address[n4] & address[n4 + n5]) == address[n4]; ++n4) {}
                if (n4 == n5) {
                    n = 2;
                }
                else {
                    n = 3;
                }
            }
            else if (this.address.length == 8 || this.address.length == 32) {
                int n2;
                int n3;
                for (n2 = 0, n3 = this.address.length / 2; n2 < n3 && (address[n2] & this.address[n2 + n3]) == this.address[n2]; ++n2) {}
                if (n2 == n3) {
                    n = 1;
                }
                else {
                    n = 3;
                }
            }
            else {
                n = 3;
            }
        }
        return n;
    }
    
    @Override
    public int subtreeDepth() throws UnsupportedOperationException {
        throw new UnsupportedOperationException("subtreeDepth() not defined for IPAddressName");
    }
}
